/*
    Copyright (C) 1998, 1999  Michael Ravits

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "include/newplayer.h"
#include "include/player_newplayer.h"
#include "include/player.h"
#include "include/control_pipe.h"
#include "include/screen.h"
#include "include/stuff.h"

static void
control_message_send_formated(void *dthread, const XA_Message *msg)
{
    control_message_post_to_command_queue(dthread, msg);
}

player::player(void)
{
	stuff	some_stuff;
	XA_Message	msg;
	
	if(player_new(&dthread, NULL) != XA_SUCCESS)
	{
		some_stuff.print_error("Can't start decoder thread.");
		exit(-1);
	}
			
	control_message_send(dthread, XA_MSG_COMMAND_OUTPUT_OPEN, "");

	msg.code		= XA_MSG_SET_OUTPUT_VOLUME;
	msg.data.volume.master_level  = 50;
	msg.data.volume.pcm_level   = 50;
	msg.data.volume.balance = 50;
	control_message_send_formated(dthread, &msg);

	player_state = IDLE;
	seek_size = 10;
	volume_size = 1;
}

player::~player(void)
{
	if(dthread)
		control_message_send(dthread, XA_MSG_COMMAND_EXIT);
}

void player::play(char *filename)
{
	if(player_state == PAUSE)
		control_message_send(dthread, XA_MSG_COMMAND_PLAY);
	else
	{
		control_message_send(dthread, XA_MSG_COMMAND_INPUT_OPEN, filename);
		control_message_send(dthread, XA_MSG_COMMAND_PLAY);
	}
}

void player::stop(void)
{
	control_message_send(dthread, XA_MSG_COMMAND_STOP);
}

void player::pause(void)
{
	control_message_send(dthread, XA_MSG_COMMAND_PAUSE);	
}

void player::seek(Direction where, class screen &scr)
{
	int	temp;
	
	if(seek_state == CANT_SEEK)
	{
		fprintf(stderr, "\n\nCan't seek!");
		return;
	}
	
	if(where == BACKWARD)
	{
		temp = -seek_size;
		if(scr.rsong_offset() - seek_size < 0)
		{
			control_message_send(dthread, XA_MSG_COMMAND_SEEK, 0, scr.rsong_range());
			return;
		}
	}
	else
		temp = seek_size;
			
	control_message_send(dthread, XA_MSG_COMMAND_SEEK, scr.rsong_offset() + temp, scr.rsong_range());
}

void player::set_volume(Direction where, class screen &scr)
{
	int		temp;
	XA_Message		msg;
	
	if(where == BACKWARD)
	{
		temp = -volume_size;
		if(scr.rvolume() + temp <= 0)
		{
			scr.set_mvb(0, 0, scr.rbalance());
			msg.code		= XA_MSG_SET_OUTPUT_VOLUME;
			msg.data.volume.master_level  = 0;
			msg.data.volume.pcm_level   = 0;
			msg.data.volume.balance = 50;
			control_message_send_formated(dthread, &msg);
			return;
		}
	}
	else
		temp = volume_size;
	if(scr.rvolume() + temp >= 100)
	{
			scr.set_mvb(100, 100, scr.rbalance());
			msg.code		= XA_MSG_SET_OUTPUT_VOLUME;
			msg.data.volume.master_level  = 100;
			msg.data.volume.pcm_level   = 100;
			msg.data.volume.balance = 50;
			control_message_send_formated(dthread, &msg);
	}
	else
	{
		scr.set_mvb(scr.rmaster_volume() + temp, scr.rvolume() + temp, scr.rbalance());
		msg.code		= XA_MSG_SET_OUTPUT_VOLUME;
		msg.data.volume.master_level  = scr.rmaster_volume() + temp;
		msg.data.volume.pcm_level   = scr.rvolume() + temp;
		msg.data.volume.balance = scr.rbalance();
		control_message_send_formated(dthread, &msg);
	}
	
	return;
}

bool player::read_message(XA_Message &msg, int &status)
{
	status = control_message_get((void *) dthread, &msg);
	if(status != 0)
		return TRUE;
	else
		return FALSE;
}

int player::rread_pipe(void)
{
	return ((XA_PipeControl *) dthread)->command_read_pipe;
}

void player::set_state(State new_state)
{
	player::player_state = new_state;
}

void player::set_seek_state(SeekState can_cant_seek)
{
	player::seek_state = can_cant_seek;
}

void player::free_decoder(void)
{
	player_delete(dthread);
}

bool player::is_playing(void)
{
	if(player_state == PLAYING)
		return TRUE;
	else
		return FALSE;
}

bool player::is_paused(void)
{
	if(player_state == PAUSE)
		return TRUE;
	else
		return FALSE;
}

bool player::is_idle(void)
{
	if(player_state == IDLE)
		return TRUE;
	else
		return FALSE;
}
